(defpackage :conv
  (:use :common-lisp)
  (:export :normalized-gaussian
	   :convolve))

(in-package :conv)

(defun gaussian (width height sigma)
  (loop
     with a = (make-array (list width height))
     for i below width
     do (loop
	   for j below height
	   do (setf (aref a i j)
		    (exp (/ (+ (expt (- i (/ (1- width) 2)) 2)
			       (expt (- j (/ (1- height) 2)) 2))
			    (* -2 (expt sigma 2))))))
     finally (return a)))

(defun sum-array (array)
  (loop
     for i below (array-total-size array)
     sum (row-major-aref array i)))

(defun normalize-array (array)
  (loop
     with a = (make-array (array-dimensions array))
     with s = (sum-array array)
     for i below (array-total-size a)
     do (setf (row-major-aref a i) (/ (row-major-aref array i) s))
     finally (return a)))

(defun normalized-gaussian (width height sigma)
  (normalize-array (gaussian width height sigma)))

(defun convolve (array kernel)
  (loop
     with a = (make-array (array-dimensions array))
     with aw = (array-dimension array 0)
     with ah = (array-dimension array 1)
     with kw = (array-dimension kernel 0)
     with kh = (array-dimension kernel 1)
     for i below aw
     do (loop
	   for j below ah
	   do (setf (aref a i j)
		    (loop
		       for k below kw
		       for m = (+ i (- k (floor kw 2)))
		       when (and (>= m 0) (< m aw))
		       sum (loop
			      for l below kh
			      for n = (+ j (- l (floor kh 2)))
			      when (and (>= n 0) (< n ah))
			      sum (* (aref array m n)
				     (aref kernel k l))))))
     finally (return a)))
