;; parameters

(defparameter *dimension* 15) ;; physical dimension
(defparameter *length* 21)
(defparameter *particles* 7)

;; libraries

(load "conf.lisp")
(require :tebdol)

(use-package '(:tensor :mps :bhm))
(setf *print-level* 2)

;; main

(defun make-mps (dimension length particles)
  (loop
     with c = (/ (1- length) 2)
     with m = (make-array length)
     for i from (- c) to c
     do
       (setf (svref m (+ i c))
	     (functional-tensor
	      (list
	       (list (make-segment
		      :numbers
		      (list
		       (cond
			 ((>= (- i) particles) 0)
			 ((>= i particles) particles)
			 ((evenp i) (/ (+ i particles -1) 2))
			 (t (/ (+ i particles) 2))))
		      :dimension
		      1))
	       (ket-physical-index dimension)
	       (list (make-segment
		      :numbers
		      (list
		       (cond
			 ((>= (- i) particles) 0)
			 ((>= i particles) (- particles))
			 ((evenp i) (- (/ (+ i particles 1) 2)))
			 (t (/ (- (+ i particles)) 2))))
		      :dimension
		      1)))
	      #'(lambda (s) (declare (ignore s)) 1)))
     finally
       (return m)))

(defparameter *mps* (make-mps *dimension* *length* *particles*))

(save-mps
 *mps*
 *dimension*
 *particles*
 (make-pathname
  :directory
  (list :relative (format nil "inputs/trotzky-l-~A-n-~A" *length* *particles*))))
