(defpackage :blas
  (:use :common-lisp :sb-alien :sb-ext :sb-sys)
  (:export :blas-float
	   :+blas-float-size+
	   :+blas-0+
	   :+blas-1+
	   :+blas-i+
	   :*blas-alien-0*
	   :*blas-alien-1*
	   :make-blas-array
	   :make-integer-array
	   :make-double-array
	   :blas-array-alien
	   :integer-array-alien
	   :double-array-alien
	   :dlamch
	   :zaxpy
	   :zgemm
	   :zgesvd
	   :zgesdd
	   :zgesvdx
	   :zheevr))

(in-package :blas)

(deftype blas-float () '(complex double-float))

(defconstant +blas-float-size+ 16)

(defconstant +blas-0+ (coerce 0 'blas-float))
(defconstant +blas-1+ (coerce 1 'blas-float))
(defconstant +blas-i+ (coerce (complex 0 1) 'blas-float))

(defparameter *blas-alien-0*
  (make-array 2 :element-type 'double-float :initial-contents '(0d0 0d0)))

(defparameter *blas-alien-1*
  (make-array 2 :element-type 'double-float :initial-contents '(1d0 0d0)))

(defmacro make-blas-array (dimensions &rest args)
  `(make-array ,dimensions ,@args :element-type 'blas-float))

(defmacro make-integer-array (dimensions &rest args)
  `(make-array ,dimensions ,@args :element-type '(signed-byte 32)))

(defmacro make-double-array (dimensions &rest args)
  `(make-array ,dimensions ,@args :element-type 'double-float))

(defun blas-array-alien (array)
  (sap-alien (vector-sap (array-storage-vector array)) (* double)))

(defun integer-array-alien (array)
  (sap-alien (vector-sap (array-storage-vector array)) (* int)))

(defun double-array-alien (array)
  (sap-alien (vector-sap (array-storage-vector array)) (* double)))

(declaim (inline dlamch zaxpy zgemm zgesvd zgesdd zgesvdx zheevr))

(define-alien-routine ("dlamch_" dlamch) double
  (cmach c-string))

(define-alien-routine ("zaxpy_" zaxpy) void
  (n int :copy)
  (za (* double))
  (zx (* double))
  (incx int :copy)
  (zy (* double))
  (incy int :copy))

(define-alien-routine ("zgemm_" zgemm) void
  (transa c-string)
  (transb c-string)
  (m int :copy)
  (n int :copy)
  (k int :copy)
  (alpha (* double))
  (a (* double))
  (lda int :copy)
  (b (* double))
  (ldb int :copy)
  (beta (* double))
  (c (* double))
  (ldc int :copy))

(define-alien-routine ("zgesvd_" zgesvd) void
  (jobu c-string)
  (jobvt c-string)
  (m int :copy)
  (n int :copy)
  (a (* double))
  (lda int :copy)
  (s (* double))
  (u (* double))
  (ldu int :copy)
  (vt (* double))
  (ldvt int :copy)
  (work (* double))
  (lwork int :copy)
  (rwork (* double))
  (info int :out))

(define-alien-routine ("zgesdd_" zgesdd) void
  (jobz c-string)
  (m int :copy)
  (n int :copy)
  (a (* double))
  (lda int :copy)
  (s (* double))
  (u (* double))
  (ldu int :copy)
  (vt (* double))
  (ldvt int :copy)
  (work (* double))
  (lwork int :copy)
  (rwork (* double))
  (iwork (* int))
  (info int :out))

(define-alien-routine ("zgesvdx_" zgesvdx) void
  (jobu c-string)
  (jobvt c-string)
  (range c-string)
  (m int :copy)
  (n int :copy)
  (a (* double))
  (lda int :copy)
  (vl double :copy)
  (vu double :copy)
  (il int :copy)
  (iu int :copy)
  (ns int :out)
  (s (* double))
  (u (* double))
  (ldu int :copy)
  (vt (* double))
  (ldvt int :copy)
  (work (* double))
  (lwork int :copy)
  (rwork (* double))
  (iwork (* int))
  (info int :out))

(define-alien-routine ("zheevr_" zheevr) void
  (jobz c-string)
  (range c-string)
  (uplo c-string)
  (n int :copy)
  (a (* double))
  (lda int :copy)
  (vl double :copy)
  (vu double :copy)
  (il int :copy)
  (iu int :copy)
  (abstol double :copy)
  (m int :in-out)
  (w (* double))
  (z (* double))
  (ldz int :copy)
  (isuppz (* int))
  (work (* double))
  (lwork int :copy)
  (rwork (* double))
  (lrwork int :copy)
  (iwork (* int))
  (liwork int :copy)
  (info int :out))
