(defpackage :util
  (:use :common-lisp)
  (:export :essentially-zerop
	   :list-permute
	   :multi-index-sequence
	   :indices-position
	   :indices-shape
	   :make-indices-permutation))

(in-package :util)

(declaim (inline essentially-zerop))

(defun essentially-zerop (number &optional (epsilon double-float-epsilon))
  (<= (abs number) epsilon))

(defun list-permute (list permutation)
  (let* ((n (length list))
         (l (make-list n)))
    (dotimes (i n l)
      (setf (nth (nth i permutation) l) (nth i list)))))

(defun multi-index-sequence (dimensions)
  (if (zerop (length dimensions))
      '(nil)
      (let ((m (multi-index-sequence (cdr dimensions))))
	(loop for i below (car dimensions) append
	     (loop for x in m collect (cons i x))))))

(defun indices-position (rank indices)
  (labels ((incp (seq &optional (i (car seq)))
	     (cond ((null seq) t)
		   ((eql (car seq) i) (incp (cdr seq) (1+ i)))
		   (t nil))))
    (let* ((s (sort (copy-list indices) #'<)))
      (when (incp s)
	(cond ((or (null s) (eql (car s) 0)) :left)
	      ((eql (1+ (car (last s))) rank) :right)
	      (t nil))))))

(defun indices-shape (indices)
  (when (car indices)
    (let ((d (loop for i in indices minimize i)))
      (mapcar #'(lambda (x) (- x d)) indices))))

(defun make-indices-permutation (rank ia ib)
  (let ((l (loop for i below rank collect i))
	(p (make-list rank :initial-element nil)))
    (mapc #'(lambda (i j)
	      (setf (nth i p) j)
	      (setf l (delete j l)))
	  ia
	  ib)
    (do ((x p (cdr x)))
    	((null x) p)
      (if (null (car x))
    	  (setf (car x) (pop l))))))
